@@ -12,16 +12,19 @@
 # limitations under the License.
 
 import collections
+import ddt
 import sys
 
 import mock
 from six import moves
+import six
 
 from cinderclient import api_versions
+from cinderclient.apiclient import base as common_base
 from cinderclient import exceptions
+from cinderclient import shell_utils
 from cinderclient import utils
 from cinderclient import base
-from cinderclient.openstack.common.apiclient import base as common_base
 from cinderclient.tests.unit import utils as test_utils
 from cinderclient.tests.unit.v2 import fakes
 
@@ -31,7 +34,7 @@ UUID = '8e8ec658-c7b0-4243-bdf8-6f7f2952c0d0'
 class FakeResource(object):
     NAME_ATTR = 'name'
 
-    def __init__(self, _id, properties):
+    def __init__(self, _id, properties, **kwargs):
         self.id = _id
         try:
             self.name = properties['name']
@@ -52,13 +55,13 @@ class FakeManager(base.ManagerWithFind):
         FakeResource('5678', {'name': '9876'})
     ]
 
-    def get(self, resource_id):
+    def get(self, resource_id, **kwargs):
         for resource in self.resources:
             if resource.id == str(resource_id):
                 return resource
         raise exceptions.NotFound(resource_id)
 
-    def list(self, search_opts):
+    def list(self, search_opts, **kwargs):
         return common_base.ListWithMeta(self.resources, fakes.REQUEST_ID)
 
 
@@ -131,6 +134,18 @@ class FindResourceTestCase(test_utils.TestCase):
         output = utils.find_resource(display_manager, 'entity_three')
         self.assertEqual(display_manager.get('4242'), output)
 
+    def test_find_by_group_id(self):
+        output = utils.find_resource(self.manager, 1234, is_group=True,
+                                     list_volume=True)
+        self.assertEqual(self.manager.get('1234', list_volume=True), output)
+
+    def test_find_by_group_name(self):
+        display_manager = FakeDisplayManager(None)
+        output = utils.find_resource(display_manager, 'entity_three',
+                                     is_group=True, list_volume=True)
+        self.assertEqual(display_manager.get('4242', list_volume=True),
+                         output)
+
 
 class CaptureStdout(object):
     """Context manager for capturing stdout from statements in its block."""
@@ -146,6 +161,61 @@ class CaptureStdout(object):
         self.read = self.stringio.read
 
 
+class BuildQueryParamTestCase(test_utils.TestCase):
+
+    def test_build_param_without_sort_switch(self):
+        dict_param = {
+            'key1': 'val1',
+            'key2': 'val2',
+            'key3': 'val3',
+        }
+        result = utils.build_query_param(dict_param, True)
+
+        self.assertIn('key1=val1', result)
+        self.assertIn('key2=val2', result)
+        self.assertIn('key3=val3', result)
+
+    def test_build_param_with_sort_switch(self):
+        dict_param = {
+            'key1': 'val1',
+            'key2': 'val2',
+            'key3': 'val3',
+        }
+        result = utils.build_query_param(dict_param, True)
+
+        expected = "?key1=val1&key2=val2&key3=val3"
+        self.assertEqual(expected, result)
+
+    def test_build_param_with_none(self):
+        dict_param = {
+            'key1': 'val1',
+            'key2': None,
+            'key3': False,
+            'key4': ''
+        }
+        result_1 = utils.build_query_param(dict_param)
+        result_2 = utils.build_query_param(None)
+
+        expected = "?key1=val1"
+        self.assertEqual(expected, result_1)
+        self.assertFalse(result_2)
+
+
+@ddt.ddt
+class ExtractFilterTestCase(test_utils.TestCase):
+
+    @ddt.data({'content': ['key1=value1'],
+               'expected': {'key1': 'value1'}},
+              {'content': ['key1={key2:value2}'],
+               'expected': {'key1': {'key2': 'value2'}}},
+              {'content': ['key1=value1', 'key2={key22:value22}'],
+               'expected': {'key1': 'value1', 'key2': {'key22': 'value22'}}})
+    @ddt.unpack
+    def test_extract_filters(self, content, expected):
+        result = shell_utils.extract_filters(content)
+        self.assertEqual(expected, result)
+
+
 class PrintListTestCase(test_utils.TestCase):
 
     def test_print_list_with_list(self):
@@ -241,12 +311,24 @@ class PrintListTestCase(test_utils.TestCase):
 """, cso.read())
 
     def test_unicode_key_value_to_string(self):
-        expected = {u'key': u'\u043f\u043f\u043f\u043f\u043f'}
-        self.assertEqual(expected, utils.unicode_key_value_to_string(expected))
+        src = {u'key': u'\u70fd\u7231\u5a77'}
+        expected = {'key': '\xe7\x83\xbd\xe7\x88\xb1\xe5\xa9\xb7'}
+        if six.PY2:
+            self.assertEqual(expected, utils.unicode_key_value_to_string(src))
+        else:
+            # u'xxxx' in PY3 is str, we will not get extra 'u' from cli
+            # output in PY3
+            self.assertEqual(src, utils.unicode_key_value_to_string(src))
 
 
 class PrintDictTestCase(test_utils.TestCase):
 
+    def test__pretty_format_dict(self):
+        content = {'key1': 'value1', 'key2': 'value2'}
+        expected = "key1 : value1\nkey2 : value2"
+        result = utils._pretty_format_dict(content)
+        self.assertEqual(expected, result)
+
     def test_print_dict_with_return(self):
         d = {'a': 'A', 'b': 'B', 'c': 'C', 'd': 'test\rcarriage\n\rreturn'}
         with CaptureStdout() as cso:
@@ -261,4 +343,20 @@ class PrintDictTestCase(test_utils.TestCase):
 | d        | test carriage |
 |          |  return       |
 +----------+---------------+
+""", cso.read())
+
+    def test_print_dict_with_dict_inside(self):
+        content = {'a': 'A', 'b': 'B', 'f_key':
+                   {'key1': 'value1', 'key2': 'value2'}}
+        with CaptureStdout() as cso:
+            utils.print_dict(content, formatters='f_key')
+        self.assertEqual("""\
++----------+---------------+
+| Property | Value         |
++----------+---------------+
+| a        | A             |
+| b        | B             |
+| f_key    | key1 : value1 |
+|          | key2 : value2 |
++----------+---------------+
 """, cso.read())
