@@ -834,6 +834,17 @@ class ShellTest(utils.TestCase):
         self.assert_called('POST', '/types/2/encryption', body=expected)
         self.assert_called_anytime('GET', '/types/2')
 
+    @ddt.data('--key-size 512 --control-location front-end',
+              '--key_size 512 --control_location front-end')  # old style
+    def test_encryption_type_create_with_args(self, arg):
+        expected = {'encryption': {'cipher': None,
+                                   'key_size': 512,
+                                   'provider': 'TestProvider',
+                                   'control_location': 'front-end'}}
+        self.run_command('encryption-type-create 2 TestProvider ' + arg)
+        self.assert_called('POST', '/types/2/encryption', body=expected)
+        self.assert_called_anytime('GET', '/types/2')
+
     def test_encryption_type_update(self):
         """
         Test encryption-type-update shell command.
@@ -1375,3 +1386,24 @@ class ShellTest(utils.TestCase):
     def test_extra_specs_list(self):
         self.run_command('extra-specs-list')
         self.assert_called('GET', '/types?is_public=None')
+
+    def test_quota_class_show(self):
+        self.run_command('quota-class-show test')
+        self.assert_called('GET', '/os-quota-class-sets/test')
+
+    def test_quota_class_update(self):
+        expected = {'quota_class_set': {'class_name': 'test',
+                                        'volumes': 2,
+                                        'snapshots': 2,
+                                        'gigabytes': 1,
+                                        'backups': 1,
+                                        'backup_gigabytes': 1,
+                                        'per_volume_gigabytes': 1}}
+        self.run_command('quota-class-update test '
+                         '--volumes 2 '
+                         '--snapshots 2 '
+                         '--gigabytes 1 '
+                         '--backups 1 '
+                         '--backup-gigabytes 1 '
+                         '--per-volume-gigabytes 1')
+        self.assert_called('PUT', '/os-quota-class-sets/test', body=expected)
