@@ -18,7 +18,6 @@ from __future__ import print_function
 import os
 import pkg_resources
 import sys
-import types
 import uuid
 
 import six
@@ -84,27 +83,6 @@ def isunauthenticated(f):
     return getattr(f, 'unauthenticated', False)
 
 
-def service_type(stype):
-    """
-    Adds 'service_type' attribute to decorated function.
-    Usage:
-        @service_type('volume')
-        def mymethod(f):
-            ...
-    """
-    def inner(f):
-        f.service_type = stype
-        return f
-    return inner
-
-
-def get_service_type(f):
-    """
-    Retrieves service type from function
-    """
-    return getattr(f, 'service_type', None)
-
-
 def _print(pt, order):
     if sys.version_info >= (3, 0):
         print(pt.get_string(sortby=order))
@@ -215,6 +193,15 @@ def build_query_param(params, sort=False):
     return query_string
 
 
+def _pretty_format_dict(data_dict):
+    formatted_data = []
+
+    for k in sorted(data_dict):
+        formatted_data.append("%s : %s" % (k, data_dict[k]))
+
+    return "\n".join(formatted_data)
+
+
 def print_dict(d, property="Property", formatters=None):
     pt = prettytable.PrettyTable([property, 'Value'], caching=False)
     pt.align = 'l'
@@ -225,18 +212,22 @@ def print_dict(d, property="Property", formatters=None):
 
         if r[0] in formatters:
             r[1] = unicode_key_value_to_string(r[1])
-
+            if isinstance(r[1], dict):
+                r[1] = _pretty_format_dict(r[1])
         if isinstance(r[1], six.string_types) and "\r" in r[1]:
             r[1] = r[1].replace("\r", " ")
         pt.add_row(r)
     _print(pt, property)
 
 
-def find_resource(manager, name_or_id):
+def find_resource(manager, name_or_id, **kwargs):
     """Helper for the _find_* methods."""
+    is_group = kwargs.pop('is_group', False)
     # first try to get entity as integer id
     try:
         if isinstance(name_or_id, int) or name_or_id.isdigit():
+            if is_group:
+                return manager.get(int(name_or_id), **kwargs)
             return manager.get(int(name_or_id))
     except exceptions.NotFound:
         pass
@@ -244,6 +235,8 @@ def find_resource(manager, name_or_id):
         # now try to get entity as uuid
         try:
             uuid.UUID(name_or_id)
+            if is_group:
+                return manager.get(name_or_id, **kwargs)
             return manager.get(name_or_id)
         except (ValueError, exceptions.NotFound):
             pass
@@ -255,12 +248,18 @@ def find_resource(manager, name_or_id):
         try:
             resource = getattr(manager, 'resource_class', None)
             name_attr = resource.NAME_ATTR if resource else 'name'
+            if is_group:
+                kwargs[name_attr] = name_or_id
+                return manager.find(**kwargs)
             return manager.find(**{name_attr: name_or_id})
         except exceptions.NotFound:
             pass
 
         # finally try to find entity by human_id
         try:
+            if is_group:
+                kwargs['human_id'] = name_or_id
+                return manager.find(**kwargs)
             return manager.find(human_id=name_or_id)
         except exceptions.NotFound:
             msg = "No %s with a name or ID of '%s' exists." % \
@@ -300,13 +299,6 @@ def _load_entry_point(ep_name, name=None):
             continue
 
 
-def retype_method(old_type, new_type, namespace):
-    for attr in namespace.values():
-        if (isinstance(attr, types.FunctionType) and
-                getattr(attr, 'service_type', None) == old_type):
-            setattr(attr, 'service_type', new_type)
-
-
 def get_function_name(func):
     if six.PY2:
         if hasattr(func, "im_class"):
