@@ -19,6 +19,7 @@ from __future__ import print_function
 import argparse
 import collections
 import os
+import sys
 
 from oslo_utils import strutils
 import six
@@ -43,6 +44,10 @@ FILTER_DEPRECATED = ("This option is deprecated and will be removed in "
            default=None,
            help='Show enabled filters for specified resource. Default=None.')
 def do_list_filters(cs, args):
+        """List enabled filters.
+
+        Symbol '~' after filter key means it supports inexact filtering.
+        """
         filters = cs.resource_filters.list(resource=args.resource)
         shell_utils.print_resource_filter_list(filters)
 
@@ -94,13 +99,14 @@ def do_list_filters(cs, args):
                   'Valid keys: %s. '
                   'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
 @utils.arg('--filters',
-           type=str,
+           type=six.text_type,
            nargs='*',
            start_version='3.33',
            metavar='<key=value>',
            default=None,
            help="Filter key and value pairs. Please use 'cinder list-filters' "
-                "to check enabled filters from server, Default=None.")
+                "to check enabled filters from server. Use 'key~=value' for "
+                "inexact filtering if the key supports. Default=None.")
 def do_backup_list(cs, args):
     """Lists all backups."""
     # pylint: disable=function-redefined
@@ -134,7 +140,7 @@ def do_backup_list(cs, args):
            action='store_true',
            help='Show detailed information about pools.')
 @utils.arg('--filters',
-           type=str,
+           type=six.text_type,
            nargs='*',
            start_version='3.33',
            metavar='<key=value>',
@@ -268,13 +274,14 @@ RESET_STATE_RESOURCES = {'volume': utils.find_volume,
            metavar='<tenant>',
            help='Display information from single tenant (Admin only).')
 @utils.arg('--filters',
-           type=str,
+           type=six.text_type,
            nargs='*',
            start_version='3.33',
            metavar='<key=value>',
            default=None,
            help="Filter key and value pairs. Please use 'cinder list-filters' "
-                "to check enabled filters from server, Default=None.")
+                "to check enabled filters from server. Use 'key~=value' "
+                "for inexact filtering if the key supports. Default=None.")
 def do_list(cs, args):
     """Lists all volumes."""
     # pylint: disable=function-redefined
@@ -497,6 +504,9 @@ def do_reset_state(cs, args):
            help=('Allow volume to be attached more than once.'
                  ' Default=False'),
            default=False)
+@utils.arg('--poll',
+           action="store_true",
+           help=('Wait for volume creation until it completes.'))
 def do_create(cs, args):
     """Creates a volume."""
 
@@ -557,6 +567,12 @@ def do_create(cs, args):
         info['readonly'] = info['metadata']['readonly']
 
     info.pop('links', None)
+
+    if args.poll:
+        timeout_period = os.environ.get("POLL_TIMEOUT_PERIOD", 3600)
+        shell_utils._poll_for_status(cs.volumes.get, volume.id, info, 'creating', ['available'],
+                timeout_period, cs.client.global_request_id, cs.messages)
+
     utils.print_dict(info)
 
 
@@ -594,6 +610,27 @@ def do_metadata(cs, args):
                                    reverse=True))
 
 
+@api_versions.wraps('3.12')
+@utils.arg('--all-tenants',
+           dest='all_tenants',
+           metavar='<0|1>',
+           nargs='?',
+           type=int,
+           const=1,
+           default=utils.env('ALL_TENANTS', default=0),
+           help='Shows details for all tenants. Admin only.')
+def do_summary(cs, args):
+    """Get volumes summary."""
+    all_tenants = args.all_tenants
+    info = cs.volumes.summary(all_tenants)
+
+    formatters = ['total_size', 'total_count']
+    if cs.api_version >= api_versions.APIVersion("3.36"):
+        formatters.append('metadata')
+
+    utils.print_dict(info['volume-summary'], formatters=formatters)
+
+
 @api_versions.wraps('3.11')
 def do_group_type_list(cs, args):
     """Lists available 'group types'. (Admin only will see private types)"""
@@ -737,10 +774,6 @@ def do_group_type_key(cs, args):
            metavar='<backup_gigabytes>',
            type=int, default=None,
            help='The new "backup_gigabytes" quota value. Default=None.')
-@utils.arg('--consistencygroups',
-           metavar='<consistencygroups>',
-           type=int, default=None,
-           help='The new "consistencygroups" quota value. Default=None.')
 @utils.arg('--groups',
            metavar='<groups>',
            type=int, default=None,
@@ -792,9 +825,9 @@ def do_quota_update(cs, args):
 @utils.arg('--image_name',
            help=argparse.SUPPRESS)
 @utils.arg('--visibility',
-           metavar='<public|private>',
-           help='Set image visibility to either public or private. '
-                'Default=private.',
+           metavar='<public|private|community|shared>',
+           help='Set image visibility to public, private, community or '
+                'shared. Default=private.',
            default='private',
            start_version='3.1')
 @utils.arg('--protected',
@@ -843,6 +876,7 @@ def do_backup_update(cs, args):
         raise exceptions.ClientException(code=1, message=msg)
 
     shell_utils.find_backup(cs, args.backup).update(**kwargs)
+    print("Request to update backup '%s' has been accepted." % args.backup)
 
 
 @api_versions.wraps('3.7')
@@ -970,13 +1004,14 @@ def do_manageable_list(cs, args):
            default=utils.env('ALL_TENANTS', default=0),
            help='Shows details for all tenants. Admin only.')
 @utils.arg('--filters',
-           type=str,
+           type=six.text_type,
            nargs='*',
            start_version='3.33',
            metavar='<key=value>',
            default=None,
            help="Filter key and value pairs. Please use 'cinder list-filters' "
-                "to check enabled filters from server, Default=None.")
+                "to check enabled filters from server. Use 'key~=value' "
+                "for inexact filtering if the key supports. Default=None.")
 def do_group_list(cs, args):
     """Lists all groups."""
     search_opts = {'all_tenants': args.all_tenants}
@@ -992,13 +1027,26 @@ def do_group_list(cs, args):
 
 
 @api_versions.wraps('3.13')
+@utils.arg('--list-volume',
+           dest='list_volume',
+           metavar='<False|True>',
+           nargs='?',
+           type=bool,
+           const=True,
+           default=False,
+           help='Shows volumes included in the group.',
+           start_version='3.25')
 @utils.arg('group',
            metavar='<group>',
            help='Name or ID of a group.')
 def do_group_show(cs, args):
     """Shows details of a group."""
     info = dict()
-    group = shell_utils.find_group(cs, args.group)
+    if getattr(args, 'list_volume', None):
+        group = shell_utils.find_group(cs, args.group,
+                                       list_volume=args.list_volume)
+    else:
+        group = shell_utils.find_group(cs, args.group)
     info.update(group._info)
 
     info.pop('links', None)
@@ -1150,9 +1198,77 @@ def do_group_update(cs, args):
         raise exceptions.ClientException(code=1, message=msg)
 
     shell_utils.find_group(cs, args.group).update(**kwargs)
+    print("Request to update group '%s' has been accepted." % args.group)
+
+
+@api_versions.wraps('3.38')
+@utils.arg('group',
+           metavar='<group>',
+           help='Name or ID of the group.')
+def do_group_enable_replication(cs, args):
+    """Enables replication for group."""
+
+    shell_utils.find_group(cs, args.group).enable_replication()
+
+
+@api_versions.wraps('3.38')
+@utils.arg('group',
+           metavar='<group>',
+           help='Name or ID of the group.')
+def do_group_disable_replication(cs, args):
+    """Disables replication for group."""
+
+    shell_utils.find_group(cs, args.group).disable_replication()
+
+
+@api_versions.wraps('3.38')
+@utils.arg('group',
+           metavar='<group>',
+           help='Name or ID of the group.')
+@utils.arg('--allow-attached-volume',
+           action='store_true',
+           default=False,
+           help='Allows or disallows group with '
+                'attached volumes to be failed over.')
+@utils.arg('--secondary-backend-id',
+           metavar='<secondary_backend_id>',
+           help='Secondary backend id. Default=None.')
+def do_group_failover_replication(cs, args):
+    """Fails over replication for group."""
+
+    shell_utils.find_group(cs, args.group).failover_replication(
+        allow_attached_volume=args.allow_attached_volume,
+        secondary_backend_id=args.secondary_backend_id)
+
+
+@api_versions.wraps('3.38')
+@utils.arg('group',
+           metavar='<group>',
+           help='Name or ID of the group.')
+def do_group_list_replication_targets(cs, args):
+    """Lists replication targets for group.
+
+    Example value for replication_targets:
+
+    .. code-block: json
+
+        {
+            'replication_targets': [{'backend_id': 'vendor-id-1',
+                                     'unique_key': 'val1',
+                                     ......},
+                                    {'backend_id': 'vendor-id-2',
+                                     'unique_key': 'val2',
+                                     ......}]
+        }
+    """
+
+    rc, replication_targets = shell_utils.find_group(
+        cs, args.group).list_replication_targets()
+    rep_targets = replication_targets.get('replication_targets')
+    if rep_targets and len(rep_targets) > 0:
+        utils.print_list(rep_targets, [key for key in rep_targets[0].keys()])
 
 
-@api_versions.wraps('3.14')
 @utils.arg('--all-tenants',
            dest='all_tenants',
            metavar='<0|1>',
@@ -1172,13 +1288,14 @@ def do_group_update(cs, args):
            help="Filters results by a group ID. Default=None. "
                 "%s" % FILTER_DEPRECATED)
 @utils.arg('--filters',
-           type=str,
+           type=six.text_type,
            nargs='*',
            start_version='3.33',
            metavar='<key=value>',
            default=None,
            help="Filter key and value pairs. Please use 'cinder list-filters' "
-                "to check enabled filters from server, Default=None.")
+                "to check enabled filters from server. Use 'key~=value' "
+                "for inexact filtering if the key supports. Default=None.")
 def do_group_snapshot_list(cs, args):
     """Lists all group snapshots."""
 
@@ -1350,6 +1467,19 @@ def do_api_version(cs, args):
     utils.print_list(response, columns)
 
 
+@api_versions.wraps("3.40")
+@utils.arg(
+    'snapshot',
+    metavar='<snapshot>',
+    help='Name or ID of the snapshot to restore. The snapshot must be the '
+         'most recent one known to cinder.')
+def do_revert_to_snapshot(cs, args):
+    """Revert a volume to the specified snapshot."""
+    snapshot = shell_utils.find_volume_snapshot(cs, args.snapshot)
+    volume = utils.find_volume(cs, snapshot.volume_id)
+    volume.revert_to_snapshot(snapshot)
+
+
 @api_versions.wraps("3.3")
 @utils.arg('--marker',
            metavar='<marker>',
@@ -1397,13 +1527,14 @@ def do_api_version(cs, args):
            help="Filters results by the message level. Default=None. "
                 "%s" % FILTER_DEPRECATED)
 @utils.arg('--filters',
-           type=str,
+           type=six.text_type,
            nargs='*',
            start_version='3.33',
            metavar='<key=value>',
            default=None,
            help="Filter key and value pairs. Please use 'cinder list-filters' "
-                "to check enabled filters from server, Default=None.")
+                "to check enabled filters from server. Use 'key~=value' "
+                "for inexact filtering if the key supports. Default=None.")
 def do_message_list(cs, args):
     """Lists all messages."""
     search_opts = {
@@ -1534,13 +1665,14 @@ def do_message_delete(cs, args):
                 "volume api version >=3.22. Default=None. "
                 "%s" % FILTER_DEPRECATED)
 @utils.arg('--filters',
-           type=str,
+           type=six.text_type,
            nargs='*',
            start_version='3.33',
            metavar='<key=value>',
            default=None,
            help="Filter key and value pairs. Please use 'cinder list-filters' "
-                "to check enabled filters from server, Default=None.")
+                "to check enabled filters from server. Use 'key~=value' "
+                "for inexact filtering if the key supports. Default=None.")
 def do_snapshot_list(cs, args):
     """Lists all snapshots."""
     # pylint: disable=function-redefined
@@ -1624,13 +1756,14 @@ def do_snapshot_list(cs, args):
            metavar='<tenant>',
            help='Display information from single tenant (Admin only).')
 @utils.arg('--filters',
-           type=str,
+           type=six.text_type,
            nargs='*',
            start_version='3.33',
            metavar='<key=value>',
            default=None,
            help="Filter key and value pairs. Please use 'cinder list-filters' "
-                "to check enabled filters from server, Default=None.")
+                "to check enabled filters from server. Use 'key~=value' "
+                "for inexact filtering if the key supports. Default=None.")
 def do_attachment_list(cs, args):
     """Lists all attachments."""
     search_opts = {
@@ -1726,7 +1859,8 @@ def do_attachment_create(cs, args):
                      'os_type': args.ostype,
                      'multipath': args.multipath,
                      'mountpoint': args.mountpoint}
-    attachment = cs.attachments.create(args.volume,
+    volume = utils.find_volume(cs, args.volume)
+    attachment = cs.attachments.create(volume.id,
                                        connector,
                                        args.server_id)
     connector_dict = attachment.pop('connection_info', None)
@@ -1816,3 +1950,44 @@ def do_version_list(cs, args):
 
     print("\nServer supported API versions:")
     utils.print_list(result, columns)
+
+
+@api_versions.wraps('3.32')
+@utils.arg('level',
+           metavar='<log-level>',
+           choices=('INFO', 'WARNING', 'ERROR', 'DEBUG',
+                    'info', 'warning', 'error', 'debug'),
+           help='Desired log level.')
+@utils.arg('--binary',
+           choices=('', '*', 'cinder-api', 'cinder-volume', 'cinder-scheduler',
+                    'cinder-backup'),
+           default='',
+           help='Binary to change.')
+@utils.arg('--server',
+           default='',
+           help='Host or cluster value for service.')
+@utils.arg('--prefix',
+           default='',
+           help='Prefix for the log. ie: "cinder.volume.drivers.".')
+def do_service_set_log(cs, args):
+    cs.services.set_log_levels(args.level, args.binary, args.server,
+                               args.prefix)
+
+
+@api_versions.wraps('3.32')
+@utils.arg('--binary',
+           choices=('', '*', 'cinder-api', 'cinder-volume', 'cinder-scheduler',
+                    'cinder-backup'),
+           default='',
+           help='Binary to query.')
+@utils.arg('--server',
+           default='',
+           help='Host or cluster value for service.')
+@utils.arg('--prefix',
+           default='',
+           help='Prefix for the log. ie: "sqlalchemy.".')
+def do_service_get_log(cs, args):
+    log_levels = cs.services.get_log_levels(args.binary, args.server,
+                                            args.prefix)
+    columns = ('Binary', 'Host', 'Prefix', 'Level')
+    utils.print_list(log_levels, columns)
